<?php
/**
* Copyright 2016 aheadWorks. All rights reserved.
* See LICENSE.txt for license details.
*/

namespace Aheadworks\OneStepCheckout\Model\Address\Form\AttributeMeta\Modifier\Attribute;

use Aheadworks\OneStepCheckout\Model\Address\Form\AttributeMeta\Modifier\ModifierInterface;
use Aheadworks\OneStepCheckout\Model\Config;
use Magento\Customer\Model\ResourceModel\Address\Attribute\Source\Country as CountrySource;
use Magento\Directory\Helper\Data as DirectoryHelper;

/**
 * Class CountryId
 * @package Aheadworks\OneStepCheckout\Model\Address\Form\AttributeMeta\Modifier\Attribute
 */
class CountryId implements ModifierInterface
{
    /**
     * @var DirectoryHelper
     */
    private $directoryHelper;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var CountrySource
     */
    private $countrySource;

    /**
     * @param DirectoryHelper $directoryHelper
     * @param Config $config
     * @param CountrySource $countrySource
     */
    public function __construct(
        DirectoryHelper $directoryHelper,
        Config $config,
        CountrySource $countrySource
    ) {
        $this->directoryHelper = $directoryHelper;
        $this->config = $config;
        $this->countrySource = $countrySource;
    }

    /**
     * {@inheritdoc}
     */
    public function modify($metadata, $addressType)
    {
        $metadata['default'] = $this->config->getDefaultCountryId() ? : $this->directoryHelper->getDefaultCountry();
        $metadata['options'] = $this->countrySource->getAllOptions();
        $metadata['options'] = $this->orderCountryOptions($metadata['options']);
        return $metadata;
    }

    /**
     * Reorder country options. Move top countries to the beginning of the list
     *
     * @param array $options
     * @return array
     */
    private function orderCountryOptions($options)
    {
        $topCountryCodes = $this->directoryHelper->getTopCountryCodes();
        if (!empty($topCountryCodes)) {
            $headOptions = [];
            $tailOptions = [[
                'value' => 'delimiter',
                'label' => '──────────',
                'disabled' => true,
            ]];

            foreach ($options as $option) {
                if (empty($option['value']) || in_array($option['value'], $topCountryCodes)) {
                    array_push($headOptions, $option);
                } else {
                    array_push($tailOptions, $option);
                }
            }
            return array_merge($headOptions, $tailOptions);
        }
        return $options;
    }
}
