<?php
/**
* Copyright 2016 aheadWorks. All rights reserved.
* See LICENSE.txt for license details.
*/

namespace Aheadworks\OneStepCheckout\Model\Layout\Processor;

use Aheadworks\OneStepCheckout\Model\Layout\DefinitionFetcher;
use Aheadworks\OneStepCheckout\Model\Layout\Processor\PaymentOptions\Merger;
use Aheadworks\OneStepCheckout\Model\Config;
use Magento\Checkout\Block\Checkout\LayoutProcessorInterface;
use Magento\Framework\Stdlib\ArrayManager;

/**
 * Class PaymentOptions
 * @package Aheadworks\OneStepCheckout\Model\Layout\Processor
 */
class PaymentOptions implements LayoutProcessorInterface
{
    /**
     * @var DefinitionFetcher
     */
    private $definitionFetcher;

    /**
     * @var ArrayManager
     */
    private $arrayManager;

    /**
     * @var Merger
     */
    private $merger;

    /**
     * @var Config
     */
    private $config;

    /**
     * @param DefinitionFetcher $definitionFetcher
     * @param ArrayManager $arrayManager
     * @param Merger $merger
     * @param Config $config
     */
    public function __construct(
        DefinitionFetcher $definitionFetcher,
        ArrayManager $arrayManager,
        Merger $merger,
        Config $config
    ) {
        $this->definitionFetcher = $definitionFetcher;
        $this->arrayManager = $arrayManager;
        $this->merger = $merger;
        $this->config = $config;
    }

    /**
     * {@inheritdoc}
     */
    public function process($jsLayout)
    {
        $discountsPath = 'components/checkout/children/payment-options/children';
        $definitionsPath = '//referenceBlock[@name="checkout.root"]/arguments/argument[@name="jsLayout"]'
            . '/item[@name="components"]/item[@name="checkout"]/item[@name="children"]'
            . '/item[@name="steps"]/item[@name="children"]/item[@name="billing-step"]'
            . '/item[@name="children"]/item[@name="payment"]/item[@name="children"]'
            . '/item[@name="afterMethods"]/item[@name="children"]';

        $discountsLayout = $this->merger->merge(
            $this->arrayManager->get($discountsPath, $jsLayout),
            $this->definitionFetcher->fetchArgs('checkout_index_index', $definitionsPath)
        );
        if (isset($discountsLayout['discount']) && !$this->config->isApplyDiscountCodeEnabled()) {
            unset($discountsLayout['discount']);
        }
        return $this->arrayManager->set($discountsPath, $jsLayout, $discountsLayout);
    }
}
