<?php
/**
* Copyright 2016 aheadWorks. All rights reserved.
* See LICENSE.txt for license details.
*/

namespace Aheadworks\OneStepCheckout\Model\Layout\Processor\PaymentOptions;

use Aheadworks\OneStepCheckout\Model\Config;

/**
 * Class Merger
 * @package Aheadworks\OneStepCheckout\Model\Layout\Processor\PaymentOptions
 */
class Merger
{
    /**
     * @var array
     */
    private $optionsComponents = ['discount'];

    /**
     * @var Config
     */
    private $config;

    /**
     * @param Config $config
     */
    public function __construct(Config $config)
    {
        $this->config = $config;
    }

    /**
     * Fetch payment options components definitions and merge into config
     *
     * @param array $config
     * @param array $sourceConfig
     * @return array
     */
    public function merge(array $config, array $sourceConfig)
    {
        foreach ($sourceConfig as $code => $configData) {
            if (in_array($code, $this->optionsComponents)
                && !in_array($code, $this->getComponentsToExclude())
            ) {
                if (!isset($config[$code])) {
                    $config[$code] = $configData;
                } else {
                    $config[$code] = array_merge($configData, $config[$code]);
                }
            }
        }
        return $config;
    }

    /**
     * Get payment options components to exclude from merge
     *
     * @return array
     */
    private function getComponentsToExclude()
    {
        $result = [];
        if (!$this->config->isApplyDiscountCodeEnabled()) {
            $result[] = 'discount';
        }
        return $result;
    }
}
