/**
* Copyright 2016 aheadWorks. All rights reserved.
* See LICENSE.txt for license details.
*/

define(
    [
        'jquery',
        'ko',
        'uiComponent',
        'Magento_Checkout/js/model/quote',
        'Magento_SalesRule/js/action/set-coupon-code',
        'Magento_SalesRule/js/action/cancel-coupon',
        'Aheadworks_OneStepCheckout/js/model/payment-option/coupon'
    ],
    function (
        $,
        ko,
        Component,
        quote,
        setCouponCodeAction,
        cancelCouponAction,
        coupon
    ) {
        'use strict';

        var totals = quote.getTotals();

        if (totals()) {
            coupon.code(totals()['coupon_code']);
        }

        return Component.extend({
            defaults: {
                template: 'Aheadworks_OneStepCheckout/sidebar/payment-option/discount',
                formSelector: '#discount-form'
            },
            couponCode: coupon.code,
            isApplied: coupon.isApplied,

            /**
             * @inheritdoc
             */
            initialize: function () {
                this._super();
                coupon.isApplied(this.couponCode() != null);
            },

            /**
             * On apply button click
             */
            onApplyClick: function() {
                if (this.isValid()) {
                    setCouponCodeAction(coupon.code(), this.isApplied);
                }
            },

            /**
             * On cancel button click
             */
            onCancelClick: function() {
                if (this.isValid()) {
                    coupon.code('');
                    cancelCouponAction(this.isApplied);
                }
            },

            /**
             * Check if coupon valid
             *
             * @returns {Boolean}
             */
            isValid: function () {
                var form = $(this.formSelector);

                return form.validation() && form.validation('isValid');
            }
        });
    }
);
